#include "Ball.h"
#include "PhysicsElement.h"

int Ball::id=0;

Ball::Ball():InertialElement(id++),mass(1.0), radius(0.1), pos_t(0,0), speed_t(0,0){}   // nobody can create a block without state

Ball::Ball(double mass, double radius, CVector position, CVector speed):InertialElement(id++),mass(mass),radius(radius){
  pos_t = position;
  speed_t = speed;
}

double Ball::getRadius() const {
  return radius;
}

CVector Ball::getSpeed() const {
  return speed_t;
}

CVector Ball::getPosition() const {
  return pos_t;
}

double Ball::getMass() const {
  return mass;
}

void Ball::computeNextState(double delta_t, MyWorld * world) {
  if(!collidingWithMe.empty()) collidingWithMe.clear();  // Assumption: on collision we only change speed.
  collidingWithMe = world->findCollidingBall(this);
  if(!collidingWithMe.empty()){ /* elastic collision */
    for(const PhysicsElement *pe:collidingWithMe){
      if(const Ball *pb=dynamic_cast<const Ball*>(pe)){
        CVector diffPosition, diffSpeed, momentumAdjustment;
        double massFactor = (2*(pb->getMass()))/(this->mass+pb->getMass());
        diffPosition=pos_t-(pb->getPosition());
        diffSpeed=speed_t-(pb->getSpeed());
        double approachingFactor=diffSpeed%diffPosition;
        double inverseModuleFactor=1/(diffPosition.moduleSquared());
        momentumAdjustment=massFactor*approachingFactor*inverseModuleFactor*diffPosition;
        speed_tPlusDelta=speed_t-momentumAdjustment;
      }
      else if(const Container *pc=dynamic_cast<const Container*>(pe)){
        collidingWall=(*pc).getCollidingWalls(this);
        CVector *speed_w=new CVector;
        *speed_w=speed_t;
        for(Wall w:collidingWall){
          *speed_w=(2*((*speed_w)%((w).getUnitary()))*((w).getUnitary()))-(*speed_w);
        }
        speed_tPlusDelta=*speed_w;
        delete speed_w;
      }
    }    
  }
  else 
    speed_tPlusDelta=speed_t; 
  pos_tPlusDelta = pos_t + speed_t*delta_t;
}

const PhysicsElement* Ball::collideWith(const Ball* pb) const { // it return itself or a physical element which is part of it 
  if (this == pb) 
    return NULL;
  CVector diffPosition=pos_t-(*pb).getPosition();
  bool areTouching=(diffPosition.module()) < (radius+(*pb).getRadius());
  if(areTouching==0) return NULL;
  CVector diffSpeed=speed_t-(*pb).getSpeed();
  bool areApproaching=(diffPosition%diffSpeed)<0;
  if (areApproaching!=0) return this;
  return NULL;
}

void Ball::updateState(){
  pos_t = pos_tPlusDelta;
  speed_t = speed_tPlusDelta;
}

string Ball::getDescription() const {
   return "Ball " + std::to_string(getId())+":x\tBall " + std::to_string(getId())+":y";
}

string Ball::getState() const{
   return std::to_string(pos_t.getX())+"\t"+std::to_string(pos_t.getY()); 
}

ostream & operator<< (ostream & os, const Ball & b) {
  os << b.pos_t;
  return os;
}